@@ -1,7 +1,7 @@
 #' Map a plan position indicator (\code{ppi})
 #'
-#' Plot a ppi on a Google Maps, OpenStreetMap, Stamen Maps or Naver Map base
-#' layer map using \link[ggmap]{ggmap}
+#' Plot a ppi on a Stamen Maps, OpenStreetMap, Google Maps or Naver Map base
+#' layer map using \link[ggmap]{ggmap}.
 #'
 #' @param x An object of class \code{ppi}.
 #' @param map  The basemap to use, result of a call to \link{download_basemap}.
@@ -10,6 +10,8 @@
 #' @param radar_size Size of the symbol indicating the radar position.
 #' @param radar_color Color of the symbol indicating the radar position.
 #' @param n_color The number of colors (>=1) to be in the palette.
+#' @param palette (Optional) character vector of hexidecimal color values defining
+#' the plot color scale, e.g. output from \link[viridisLite]{viridis}
 #' @param xlim Range of x values to plot (degrees longitude), as atomic
 #' vector of length 2.
 #' @param ylim Range of y values to plot (degrees latitude), as an atomic
@@ -30,6 +32,7 @@
 #' \code{summary(x)}. Commonly available parameters are:
 #' \describe{
 #'  \item{"\code{DBZH}", "\code{DBZ}"}{(Logged) reflectivity factor [dBZ]}
+#'  \item{"\code{TH}", "\code{T}"}{(Logged) uncorrected reflectivity factor [dBZ]}
 #'  \item{"\code{VRADH}", "\code{VRAD}"}{Radial velocity [m/s]. Radial
 #'  velocities towards the radar are negative, while radial velocities away
 #'  from the radar are positive}
@@ -43,34 +46,36 @@
 #' \href{https://github.com/adokter/vol2bird/blob/master/doc/OPERA2014_O4_ODIM_H5-v2.2.pdf}{ODIM specification}.
 #'
 #' @export
-#'
 #' @examples
 #' # load an example scan:
 #' data(example_scan)
 #' # make ppi's for all scan parameters in the scan
 #' ppi <- project_as_ppi(example_scan)
+#' \dontrun{
 #' # grab a basemap that matches the extent of the ppi:
-#' \dontrun{
-#' basemap <- download_basemap(ppi)
-#' }
+#' # using a gray-scale basemap:
+#' basemap <- download_basemap(ppi, maptype = "toner-lite")
+#'
 #' # map the radial velocity scan parameter onto the basemap:
-#' \dontrun{
 #' map(ppi, map = basemap, param = "VRADH")
-#' }
+#'
 #' # extend the plotting range of velocities, from -50 to 50 m/s:
-#' \dontrun{
 #' map(ppi, map = basemap, param = "VRADH", zlim = c(-50, 50))
-#' }
-#' # give the data less transparency:
-#' \dontrun{
-#' map(ppi, map = basemap, alpha = 0.9)
-#' }
+#'
+#' # map the reflectivity on a terrain basemap:
+#' basemap <- download_basemap(ppi, maptype = "terrain")
+#' map(ppi, map = basemap, param = "DBZH")
+#'
+#' # change the color palette, e.g. Viridis colors:
+#' map(ppi, map = basemap, param = "DBZH", palette = viridis::viridis(100), zlim=c(-10,10))
+#'
+#' # give the data more transparency:
+#' map(ppi, map = basemap, param = "DBZH", alpha = 0.3)
+#'
 #' # change the appearance of the symbol indicating the radar location:
-#' \dontrun{
-#' map(ppi, map = basemap, radar_size = 5, radar_color = "green")
-#' }
+#' map(ppi, map = basemap, radar_size = 5, radar_color = "blue")
+#'
 #' # crop the map:
-#' \dontrun{
 #' map(ppi, map = basemap, xlim = c(12.4, 13.2), ylim = c(56, 56.5))
 #' }
 map <- function(x, ...) {
@@ -82,7 +87,8 @@ map <- function(x, ...) {
 map.ppi <- function(x, map, param, alpha = 0.7, xlim, ylim,
                     zlim = c(-20, 20), ratio, radar_size = 3,
                     radar_color = "red", n_color = 1000,
-                    radar.size = 3, radar.color = "red", n.color = 1000, ...) {
+                    radar.size = 3, radar.color = "red",
+                    n.color = 1000, palette = NA, ...) {
 
   # deprecate function arguments
   if (!missing(radar.size)) {
@@ -106,6 +112,8 @@ map.ppi <- function(x, map, param, alpha = 0.7, xlim, ylim,
 
   stopifnot(inherits(x, "ppi"))
 
+  if (hasArg("quantity")) stop("unknown function argument 'quantity`. Did you mean `param`?")
+
   if (missing(param)) {
     if ("DBZH" %in% names(x$data)) {
       param <- "DBZH"
@@ -119,11 +127,14 @@ map.ppi <- function(x, map, param, alpha = 0.7, xlim, ylim,
     )
   }
   if (missing(zlim)) {
-    zlim <- get_zlim(param)
+    zlim <- get_zlim(param, zlim)
   }
   if (!(param %in% names(x$data))) {
     stop(paste("no scan parameter '", param, "' in this ppi", sep = ""))
   }
+  if (is.na(raster::crs(x$data))) {
+    stop("Not a projected ppi, map() expects a ppi generated by project_as_ppi() with argument project=TRUE")
+  }
   if (!attributes(map)$ppi) {
     stop("Not a ppi map, use download_basemap() to download a map.")
   }
@@ -132,6 +143,17 @@ map.ppi <- function(x, map, param, alpha = 0.7, xlim, ylim,
     stop("Not a basemap for this radar location.")
   }
 
+  # set color scales and palettes
+  if (!are_equal(palette, NA)) {
+    if(!(is.character(palette) && length(palette) > 1)) stop("palette should be a character vector with hex color values")
+    n_color = length(palette)
+    colorscale <- color_palette_to_scale_fill(param, zlim, palette, na.value = "transparent")
+  }
+  else{
+    palette <- color_palette(param = param, n_color = n_color, alpha = alpha)
+    colorscale <- color_scale(param, zlim)
+  }
+
   # extract the scan parameter
   data <- do.call(function(y) x$data[y], list(param))
   wgs84 <- CRS("+proj=longlat +datum=WGS84")
@@ -172,39 +194,25 @@ map.ppi <- function(x, map, param, alpha = 0.7, xlim, ylim,
     data[index, ]$z <- zlim[2]
   }
 
-  # rasterize
-  r <- raster::rasterize(data[, 2:3], r, data[, 1])
-  # assign colors
-  if (param %in% c("VRADH", "VRADV", "VRAD")) {
-    cols <- add_color_transparency(
-      colorRampPalette(
-        colors = c("blue", "white", "red"),
-        alpha = TRUE
-      )(n_color),
-      alpha = alpha
-    )
-  } else {
-    cols <- add_color_transparency(
-      colorRampPalette(
-        colors = c(
-          "lightblue", "darkblue", "green",
-          "yellow", "red", "magenta"
-        ),
-        alpha = TRUE
-      )(n_color),
-      alpha = alpha
-    )
+  # rasterize data on the raster, if there is valid data
+  if(FALSE %in% is.na(data[,1])){
+    r <- raster::rasterize(data[, 2:3], r, data[, 1])
+  } else{
+    raster::values(r) <- NA
   }
 
+  # function to convert values to hex color strings
   col_func <- function(value, lim) {
     output <- rep(0, length(value))
     output <- round((value - lim[1]) / (lim[2] - lim[1]) * n_color)
     output[output > n_color] <- n_color
     output[output < 1] <- 1
-    return(cols[output])
+    return(palette[output])
   }
 
+  # convert data values to hex color string values.
   r@data@values <- col_func(r@data@values, zlim)
+
   # these declarations prevent generation of NOTE "no visible binding for
   # global variable" during package Check
   lon <- lat <- y <- z <- NA
@@ -224,8 +232,6 @@ map.ppi <- function(x, map, param, alpha = 0.7, xlim, ylim,
     size = radar_size,
     data = data.frame(lon = x$geo$lon, lat = x$geo$lat)
   )
-  # colorscale
-  colorscale <- color_scale(param, zlim)
   # bounding box
   bboxlatlon <- attributes(map)$geo$bbox
   # remove dimnames, otherwise ggmap will give a warning message below:
@@ -243,12 +249,3 @@ map.ppi <- function(x, map, param, alpha = 0.7, xlim, ylim,
   )
   suppressWarnings(mymap)
 }
-
-
-get_zlim <- function(param) {
-  if (param %in% c("DBZH", "DBZV", "DBZ")) return(c(-20, 30))
-  if (param %in% c("VRADH", "VRADV", "VRAD")) return(c(-20, 20))
-  if (param == "RHOHV") return(c(0.4, 1))
-  if (param == "ZDR") return(c(-5, 8))
-  if (param == "PHIDP") return(c(-200, 200))
-}
